
package CustomPaperSize;

require(Exporter);
@ISA=qw(Exporter);

use strict;
use POSIX;
use Gtk;

use vars qw (@EXPORT);

# export functions
@EXPORT= qw ( messageBox  customPaperSizeDlg  custompageListDlg );

# spin metrix for dots
my $STEP_INCREMENT_DOTS=10;
my $PAGE_INCREMENT_DOTS=50;
my $PAGE_SIZE_DOTS=0;

# spin metrix for inches
my $STEP_INCREMENT_INCHES=0.1;
my $PAGE_INCREMENT_INCHES=0.5;
my $PAGE_SIZE_INCHES=0;

# spin metrix for inches
my $STEP_INCREMENT_MIL=5;
my $PAGE_INCREMENT_MIL=10;
my $PAGE_SIZE_MIL=0;



# define conversion constants
my $INCH2MIL   = 25.4;
my $INCH2DOT   = 72;

# parameters 
# custompapersize name ( index )
# width value
# length value
# min width
# max width
# min Length
# max Length


# the data floder
#     where all data files are stored
#     this is the parent folder for :
#                          PAF folder
#                          <queue_name>.dat 
#                          ...  
my $ibmDataDir = '/etc/ibmlp/data';

# PAF folder
my $ibmPafDir = "${ibmDataDir}/pafs";

# the object which contains the translated messages
my $transObj = undef; 

# files which contain messages  ( PAF format )
my $ibmMessagesPaf = "${ibmPafDir}/messages.paf";

# setup message translation
$transObj = new Support::messages($ibmMessagesPaf);



#################################################################################
#################################################################################
##
##  subroutine:    messageBox ( EXPORTED )
##   
##  description:  function that show a message box
##
##  parameters:   msgKind  -  the title of the window
##                msgText  -  the message in the window
##
##  return:       ( no value ) 
##
#################################################################################
#################################################################################
sub messageBox {
    my $msgKind = shift;
    my $msgText = shift;

#   Create the Message Dialog Window
    my $window  = new Gtk::Window('toplevel');
    show $window;

    $window->set_title($msgKind);
    $window->set_position('center');
    $window->set_default_size(300,80);
    $window->set_modal(1);

#    Create VBox
    my $vbox = new Gtk::VBox(0,2);
    $window->add($vbox);
    show $vbox;

#   Create the Message Label
    my $label = new Gtk::Label($msgText);
    show $label;

#   Add Message Label to the VBox
    $vbox->pack_start($label,1,1,2);

#   Create Button HBox
    my $hbox    = new Gtk::HBox(1,2);
    show $hbox;
    $vbox->pack_start($hbox,0,0,2);

    my $okButton = new Gtk::Button("   ".$transObj->translate("ok")."   ");
    show $okButton;
    $hbox->pack_start($okButton,0,0,2);
    $okButton->signal_connect("clicked",sub{$window->destroy});
}


#################################################################################
#
#  subroutine:    round         ( FOR LOCAL USE )
#   
#  description:   compute the aprox. of an integer form a real
#                          2.2 --->  2
#                          2.5 --->  3
#                          2.8 --->  3 
#
#  parameters:    value - the real value
#
#  return:       the aprox. value
#
#################################################################################
sub round {
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 1 parameter(s) .\n");
    }
    my $value= shift;
    my $retValue=0;
    if($value-floor($value)>=0.5) {
	$retValue=ceil($value);
    } else {
	$retValue=floor($value);
    }

}

#################################################################################
#
#  subroutine:    convertMetrics                 ( FOR LOCAL USE )
#   
#  description:   convert a value from a metrics to another
#                 available metrics : 
#                                      "dots"
#                                      "inches"
#                                      "millimeters"
#
#  parameters:  1 - oldUNIT
#               2 - newUNIT
#               3 - the value in oldUNIT metrics
#                  
#
#  return:      the value in the newUNIT metrics 
#
#################################################################################
sub convertMetrics {
    if ( @_ < 3 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 3 parameter(s) .\n");
    }
    my $oldUNIT = shift;
    my $newUNIT = shift;
    my $value   = shift;

    my $internValue = 0;

    # convert old unit to DOTS
    if($oldUNIT eq "millimeters") {
	$internValue=$value*$INCH2DOT/$INCH2MIL;
    } elsif ( $oldUNIT eq "inches" ) {
	$internValue=$value*$INCH2DOT;
    } else {
	$internValue=$value;
    }

    my $retValue = 0;

    $retValue=$internValue;

    if($newUNIT eq "inches") {
	$retValue = $retValue/$INCH2DOT;
	$retValue = round ($retValue*100)/100; 
    } elsif ( $newUNIT eq "millimeters" ) {
	$retValue =  $retValue*$INCH2MIL/$INCH2DOT;
	$retValue = round($retValue*10)/10;
    } else {
	$retValue = round($retValue);
    }

}


#################################################################################
#################################################################################
##
##  subroutine:   customPaperSizeDlg ( EXPORTED )
##   
##  description:   the dialog window for defineing the a Custom Page Size
##
##  parameters:  1.  name      - the Name of the Paper Size
##               2.  width     - the current width of the Paper Size
##               3.  length    - the current length of the Paper Size
##               4.  printer   - the hash of the printer assoc. with the queue
##               5.  customHash- the hash with the defined custom papers
##               6.  the CList ref of custom paper sizes 
##               7.  the position of the custom paper size if edited !!!
##                  
##
##  return:      ( no value )
##
#################################################################################
#################################################################################
sub customPaperSizeDlg {
    
######
#         THE PARAMETERS
#
	
    if ( @_ < 7 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 7 parameter(s) .\n");
    }

    my $name             = shift;
    my $width            = shift;
    my $length           = shift;
    my $printer          = shift;
    my $customPaperSizes = shift;
    my $papersCList      = shift;
    my $row              = shift; # a ref. to a scalar which conatins
                                  # the current selected row of the 
                                  # CList

    my $oldName          = $name;



    my $minWidth     = $printer->getCustomPaperSizeParameter("MinWidth");
    my $maxWidth     = $printer->getCustomPaperSizeParameter("MaxWidth");
    my $minLength    = $printer->getCustomPaperSizeParameter("MinLength");
    my $maxLength    = $printer->getCustomPaperSizeParameter("MaxLength");

    if( ! defined $length ) {
	$length =  $minLength ;
    }

    if( ! defined $width ) {
	$width =  $minWidth ;
    }

 
    # helper structure
    my %helper;

    $helper{"width"}       =$width;
    $helper{"length"}      =$length;
    $helper{"minWidth"}    =$minWidth;
    $helper{"maxWidth"}    =$maxWidth;
    $helper{"minLength"}   =$minLength;
    $helper{"maxLength"}   =$maxLength;
    $helper{"oldName"}     =$oldName;

    $helper{"metric"}      =      "dots";
    $helper{"printer"}     =      $printer;
    $helper{"customPaperSizes"} = $customPaperSizes;
    $helper{"papersCList"}=       $papersCList;
    $helper{"row"}        =       $row;



######
#         DEFINE THE CONTAINTER WINDOW
#
    my $window = new Gtk::Window('toplevel');
    $helper{"window"}      =$window;
    $window->set_title($transObj->translate("DCPS_Dlg_Title"));
    $window->set_position('center');
    $window->set_modal(1);
    $window->show;       
    $window->set_default_size(350,200);

#####
#       DEFINE THE TOP V. BOX THAT CONTAIS ALL CONTROLS
#
    my $topBox = new Gtk::VBox(0,5);
    $window->add($topBox);
    $topBox->show;
    $topBox->set_border_width(20);

#####
#     DEFINE THE H. BOX THAT CONTAINS THE CONTROLS FOR DEFINITION OF QUEUE NAME
#
    my $nameBox = new Gtk::HBox(0,2);
    $nameBox->show;
#    $nameBox->set_border_width(10);
    $topBox->pack_start($nameBox,1,1,10);

#####
#     DEFINE THE NAME CONTROLS
#    
    # label
    my $nameLabel = new Gtk::Label($transObj->translate("DCPS_Dlg_Paper_Label"));
    $nameLabel->show;
    $nameBox->pack_start($nameLabel,1,1,1);
    # text entry
    my $nameEntry = new Gtk::Entry;    
    $helper{"nameEntry"}   =$nameEntry;
    $nameEntry->show;
    $nameEntry->set_text($name);
    $nameBox->pack_start($nameEntry,1,1,1);

#####
#     DEFINE THE H. BOX THAT CONTAINS THE WIDTH CONTROLS
#
    my $widthBox = new Gtk::HBox(0,2);
    $widthBox->show;

#####
#     DEFINE THE  WIDTH CONTROLS
#
    # width label 
    my $widthLabel = new Gtk::Label($transObj->translate("DCPS_Dlg_Width"));
    $widthLabel->show;
    $widthBox->pack_start($widthLabel,1,1,1);
    
    # width edit
    my $widthAdj = new Gtk::Adjustment $width, # the current value
                                       $minWidth, # the min value
                                       $maxWidth, # the max value
                                       $STEP_INCREMENT_DOTS, # step_increment
                                       $PAGE_INCREMENT_DOTS, # page_increment
                                       $PAGE_SIZE_DOTS     ; # page_size

    my $widthSpin = new Gtk::SpinButton $widthAdj, 0, 0;
    $helper{"widthSpin"}   =$widthSpin;
    $widthSpin->show;
    $widthSpin->set_user_data(\%helper); 
    $widthSpin->set_wrap(1);
    $widthBox->pack_start($widthSpin, 1, 1, 1);

    # width unit measure
    my $unitWidthLabel= new Gtk::Label("( $minWidth - $maxWidth ".
				       $transObj->translate("DCPS_Dlg_Dots").
				       " )" );
    $helper{"widthLabel"}  =$unitWidthLabel;
    $unitWidthLabel->show;
    $widthBox->pack_start($unitWidthLabel,1,1,1);


#####
#     DEFINE THE H. BOX THAT CONTAINS THE LENGTH CONTROLS
#
    my $lengthBox = new Gtk::HBox(0,2);
    $lengthBox->show;


#####
#     DEFINE THE  LENGTH CONTROLS
#
    # length label 
    my $lengthLabel = new Gtk::Label($transObj->translate("DCPS_Dlg_Length"));
    $lengthLabel->show;
    $lengthBox->pack_start($lengthLabel,1,1,1);

    # length edit
   my $lengthAdj = new Gtk::Adjustment  $length, # the current value
                                        $minLength, # the min value
                                        $maxLength, # the max value
                                        $STEP_INCREMENT_DOTS, # step_increment
                                        $PAGE_INCREMENT_DOTS, # page_increment
                                        $PAGE_SIZE_DOTS     ; # page_size
    my $lengthSpin = new Gtk::SpinButton $lengthAdj, 0, 0;
    $helper{"lengthSpin"}  =$lengthSpin;
    $lengthSpin->show;
    $lengthSpin->set_user_data(\%helper); 
    $lengthSpin->set_wrap(1);
    $lengthBox->pack_start($lengthSpin, 1, 1, 1);

    # length unit measure
    my $unitLengthLabel= new Gtk::Label("( $minLength - $maxLength ".
				       $transObj->translate("DCPS_Dlg_Dots").
				       " )" );
  
    $helper{"lengthLabel"} =$unitLengthLabel;
    $unitLengthLabel->show;
    $lengthBox->pack_start($unitLengthLabel,1,1,1);


#####
#         DEFINE THE RADIO BUTTONS
#
    # create the frame for the buttons
    my $radioFrame = new Gtk::Widget("Gtk::Frame",
                    label => $transObj->translate("DCPS_Dlg_Units"),
                    border_width => 1,
                    visible => 1);
    $radioFrame->show;
    $topBox->pack_start($radioFrame,1,1,1);
    $topBox->pack_start($widthBox,1,1,1);
    $topBox->pack_start($lengthBox,1,1,1);


    # the vbox for radio buttons
    my $radioButtonsBox=new Gtk::VBox(0,5);
    $radioButtonsBox->show;
    $radioFrame->add($radioButtonsBox);

    # pels radio button
    my $pelsButton = new Gtk::RadioButton ($transObj->translate("DCPS_Dlg_Dots"));
    $pelsButton->signal_connect("clicked",\&dotsSelect);
    $pelsButton->set_user_data(\%helper);
    $pelsButton->show;
    $radioButtonsBox->pack_start($pelsButton ,0,0,0);

    # inches  radio button
    my $inchesButton = new Gtk::RadioButton ($transObj->translate("DCPS_Dlg_Inches"),
					     $pelsButton);
    $inchesButton->signal_connect("clicked",\&inchesSelect);
    $inchesButton->set_user_data(\%helper);
    $inchesButton->show;
    $radioButtonsBox->pack_start($inchesButton ,0,0,0);

    # millimeters radio button
    my $millButton = new Gtk::RadioButton ($transObj->translate("DCPS_Dlg_Millimeters")
					   ,$inchesButton);
    $millButton->signal_connect("clicked",\&milSelect);
    $millButton->set_user_data(\%helper);
    $millButton->show;
    $radioButtonsBox->pack_start($millButton ,0,0,0);



#####
#   DEFINE THE  H. BOX OF BUTTONS
#
    my $buttonsBox = new Gtk::HBox(1,1);
    $buttonsBox->show;
    $topBox->pack_start($buttonsBox,1,1,1);

#####
#      DEFINE THE BUTTONS
#
    # ok button
    my $okButton = new Gtk::Button("  ".
				   $transObj->translate("ok").
				   "  ");
    $okButton->set_user_data(\%helper);
    $okButton->signal_connect("clicked",\&okCustomPaperSizeDlgClicked);    
    $okButton->show;
    $buttonsBox->pack_start($okButton,0,0,2);
    # cancel button
    my $cancelButton = new Gtk::Button($transObj->translate("cancel"));
    $cancelButton->show;
    $cancelButton->set_user_data(\%helper);
    $cancelButton->signal_connect("clicked",\&cancelCustomPaperSizeDlgClicked);    
    $buttonsBox->pack_start($cancelButton,0,0,2);


}

#################################################################################
#
#  subroutine:    dotsSelect               ( FOR LOCAL USE )
#   
#  description:   callback function for Dots radio button
#
#  parameters:    widget   - a ref to the Dots radio button widget
#
#  return:        ( no value ) 
#
#################################################################################
sub dotsSelect {
    my $widget = shift;
    if(! $widget->active ) {
	return;
    }
    my $helper = $widget->get_user_data;
    my $widthSpin  = $helper->{"widthSpin"}; 
    my $lengthSpin = $helper->{"lengthSpin"};
    my $minWidth   = $helper->{"minWidth"};
    my $maxWidth   = $helper->{"maxWidth"}; 
    my $minLength  = $helper->{"minLength"};
    my $maxLength  = $helper->{"maxLength"};

    # get the value setted in spins 
    my $widthValue =  $widthSpin->get_value_as_float;
    my $lengthValue = $lengthSpin->get_value_as_float;
    # convert it to dots
    $widthValue = convertMetrics($helper->{metric},"dots",$widthValue);
    $lengthValue= convertMetrics($helper->{metric},"dots",$lengthValue);
    # set the new values
    $helper->{"width"}=$widthValue;
    $helper->{"length"}=$lengthValue;

    # convert them to the new measure unit: DOTS
    my $newMetricWidthValue  =  convertMetrics("dots","dots",$widthValue);
    my $newMetricLengthValue =  convertMetrics("dots","dots",$lengthValue);
    # set the spin controls with the new values and metrics
    # width
    my $newWidthAdjustemnt = new  Gtk::Adjustment  
	                      $newMetricWidthValue, # the current value
                              convertMetrics("dots","dots",$minWidth), # the min value
                              convertMetrics("dots","dots",$maxWidth), # the max value
                              $STEP_INCREMENT_DOTS, # step_increment
                              $PAGE_INCREMENT_DOTS, # page_increment
                              $PAGE_SIZE_DOTS     ; # page_size
    $widthSpin->set_adjustment( $newWidthAdjustemnt);
    $widthSpin->set_digits(0); # no floating point ...
    $widthSpin->set_value($newMetricWidthValue);
    # length
    my $newLengthAdjustemnt = new  Gtk::Adjustment  
	                      $newMetricLengthValue, # the current value
                              convertMetrics("dots","dots",$minLength), # the min value
                              convertMetrics("dots","dots",$maxLength), # the max value
                              $STEP_INCREMENT_DOTS, # step_increment
                              $PAGE_INCREMENT_DOTS, # page_increment
                              $PAGE_SIZE_DOTS     ; # page_size
    $lengthSpin->set_adjustment( $newLengthAdjustemnt);
    $lengthSpin->set_digits(0); # no floating point ...
    $lengthSpin->set_value($newMetricLengthValue);
    # set in the helper the new metrics
    $helper->{"metric"}="dots";

    # set the new labels for width and length
    my $unitWidthLabel=$helper->{"widthLabel"};
    my $widthText="( ".
	convertMetrics("dots","dots",$minWidth).
	    " - ".
		convertMetrics("dots","dots",$maxWidth).
		    " ".$transObj->translate("DCPS_Dlg_Dots").
			" )";
    $unitWidthLabel->set($widthText);
    my $unitLengthLabel=$helper->{"lengthLabel"};
    my $lengthText="( ".
	convertMetrics("dots","dots",$minLength).
	    " - ".
		convertMetrics("dots","dots",$maxLength).
		    " ".$transObj->translate("DCPS_Dlg_Dots").
			" )";
    $unitLengthLabel->set($lengthText);
}

#################################################################################
#
#  subroutine:    inchesSelect           ( FOR LOCAL USE )
#   
#  description:   callback function for Inches radio button
#
#  parameters:    widget   - a ref to the Inches radio button widget
#
#  return:        ( no value ) 
#
#################################################################################
sub inchesSelect {
    my $widget = shift;
    if(! $widget->active ) {
	return;
    }
    my $helper = $widget->get_user_data;
    my $widthSpin  = $helper->{"widthSpin"}; 
    my $lengthSpin = $helper->{"lengthSpin"};
    my $minWidth   = $helper->{"minWidth"};
    my $maxWidth   = $helper->{"maxWidth"}; 
    my $minLength  = $helper->{"minLength"};
    my $maxLength  = $helper->{"maxLength"};

    # get the value setted in spins 
    my $widthValue =  $widthSpin->get_value_as_float;
    my $lengthValue = $lengthSpin->get_value_as_float;
    # convert it to dots
    $widthValue = convertMetrics($helper->{metric},"dots",$widthValue);
    $lengthValue= convertMetrics($helper->{metric},"dots",$lengthValue);
    # set the new values
    $helper->{"width"}=$widthValue;
    $helper->{"length"}=$lengthValue;

    # convert them to the new measure unit: DOTS
    my $newMetricWidthValue  =  convertMetrics("dots","inches",$widthValue);
    my $newMetricLengthValue =  convertMetrics("dots","inches",$lengthValue);
    # set the spin controls with the new values and metrics
    # width
   my $newWidthAdjustemnt = new  Gtk::Adjustment  
	                      $newMetricWidthValue, # the current value
                              convertMetrics("dots","inches",$minWidth), # the min value
                              convertMetrics("dots","inches",$maxWidth), # the max value
                              $STEP_INCREMENT_INCHES, # step_increment
                              $PAGE_INCREMENT_INCHES, # page_increment
                              $PAGE_SIZE_INCHES     ; # page_size
    $widthSpin->set_adjustment( $newWidthAdjustemnt);
    $widthSpin->set_digits(2); # 2 digits floating point
    $widthSpin->set_value($newMetricWidthValue);
    # length
    my $newLengthAdjustemnt = new  Gtk::Adjustment  
	                      $newMetricLengthValue, # the current value
                              convertMetrics("dots","inches",$minLength), # the min value
                              convertMetrics("dots","inches",$maxLength), # the max value
                              $STEP_INCREMENT_INCHES, # step_increment
                              $PAGE_INCREMENT_INCHES, # page_increment
                              $PAGE_SIZE_INCHES     ; # page_size
    $lengthSpin->set_adjustment( $newLengthAdjustemnt);
    $lengthSpin->set_digits(2); # 2 digits floating point
    $lengthSpin->set_value($newMetricLengthValue);
    # set in the helper the new metrics
    $helper->{"metric"}="inches";

    # set the new labels for width and length
    my $unitWidthLabel=$helper->{"widthLabel"};
    my $widthText="( ".
	convertMetrics("dots","inches",$minWidth).
	    " - ".
		convertMetrics("dots","inches",$maxWidth).
		    " ".$transObj->translate("DCPS_Dlg_Inches").
			" )";
    $unitWidthLabel->set($widthText);
    my $unitLengthLabel=$helper->{"lengthLabel"};
    my $lengthText="( ".
	convertMetrics("dots","inches",$minLength).
	    " - ".
		convertMetrics("dots","inches",$maxLength).
		    " ".$transObj->translate("DCPS_Dlg_Inches").
			" )";
    $unitLengthLabel->set($lengthText);

}

#################################################################################
#
#  subroutine:    milSelect        ( FOR LOCAL USE )
#   
#  description:   callback function for Millimeters radio button
#
#  parameters:    widget   - a ref to the Millimeters radio button widget
#
#  return:        ( no value ) 
#
#################################################################################
sub milSelect {
    my $widget = shift;
    if(! $widget->active ) {
	return;
    }
    my $helper = $widget->get_user_data;
    my $widthSpin  = $helper->{"widthSpin"}; 
    my $lengthSpin = $helper->{"lengthSpin"};
    my $minWidth   = $helper->{"minWidth"};
    my $maxWidth   = $helper->{"maxWidth"}; 
    my $minLength  = $helper->{"minLength"};
    my $maxLength  = $helper->{"maxLength"};

    # get the value setted in spins 
    my $widthValue =  $widthSpin->get_value_as_float;
    my $lengthValue = $lengthSpin->get_value_as_float;
    # convert it to dots
    $widthValue = convertMetrics($helper->{metric},"dots",$widthValue);
    $lengthValue= convertMetrics($helper->{metric},"dots",$lengthValue);
    # set the new values
    $helper->{"width"}=$widthValue;
    $helper->{"length"}=$lengthValue;

    # convert them to the new measure unit: MILLIMETERS
    my $newMetricWidthValue  =  convertMetrics("dots","millimeters",$widthValue);
    my $newMetricLengthValue =  convertMetrics("dots","millimeters",$lengthValue);
    # set the spin controls with the new values and metrics
    # width
    my $newWidthAdjustemnt = new  Gtk::Adjustment  
	                      $newMetricWidthValue, # the current value
                              convertMetrics("dots","millimeters",$minWidth), # the min value
                              convertMetrics("dots","millimeters",$maxWidth), # the max value
                              $STEP_INCREMENT_INCHES, # step_increment
                              $PAGE_INCREMENT_INCHES, # page_increment
                              $PAGE_SIZE_INCHES     ; # page_size
    $widthSpin->set_adjustment( $newWidthAdjustemnt );
    $widthSpin->set_digits(1); # 1 digits floating point
    $widthSpin->set_value($newMetricWidthValue);
    # length
    my $newLengthAdjustemnt = new  Gtk::Adjustment  
	                      $newMetricLengthValue, # the current value
                              convertMetrics("dots","millimeters",$minLength), # the min value
                              convertMetrics("dots","millimeters",$maxLength), # the max value
                              $STEP_INCREMENT_MIL, # step_increment
                              $PAGE_INCREMENT_MIL, # page_increment
                              $PAGE_SIZE_MIL     ; # page_size
    $lengthSpin->set_adjustment( $newLengthAdjustemnt );
    $lengthSpin->set_digits(1); # 1 digits floating point
    $lengthSpin->set_value($newMetricLengthValue);
    # set in the helper the new metrics
    $helper->{"metric"}="millimeters";

    # set the new labels for width and length
    my $unitWidthLabel=$helper->{"widthLabel"};
    my $widthText="( ".
	convertMetrics("dots","millimeters",$minWidth).
	    " - ".
		convertMetrics("dots","millimeters",$maxWidth).
		    " ".$transObj->translate("DCPS_Dlg_Millimeters").
			" )";
    $unitWidthLabel->set($widthText);
    my $unitLengthLabel=$helper->{"lengthLabel"};
    my $lengthText="( ".
	convertMetrics("dots","millimeters",$minLength).
	    " - ".
		convertMetrics("dots","millimeters",$maxLength).
		    " ".$transObj->translate("DCPS_Dlg_Millimeters").
			" )";
    $unitLengthLabel->set($lengthText);



}


#################################################################################
#
#  subroutine:    okClicked       ( FOR LOCAL USE )
#   
#  description:   callback function for OK button
#
#  parameters:    widget   - a ref to the OK  widget
#
#  return:        ( no value ) 
#
#################################################################################
sub okCustomPaperSizeDlgClicked {
    my $okButton = shift;
    my $helper       = $okButton->get_user_data;
    my $window       = $helper->{"window"};
    my $nameEntry    = $helper->{"nameEntry"};
    my $printer      = $helper->{"printer"};
    my $customPaperSizes =$helper->{"customPaperSizes"};
    my $widthSpin  = $helper->{"widthSpin"}; 
    my $lengthSpin = $helper->{"lengthSpin"};

    # check if its something in the entry field
    my $paperName    = $nameEntry->get_text;
    if(! $paperName ) {
	messageBox( $transObj->translate("MSG_ERROR"),
		    $transObj->translate("DCPS_Dlg_Paper_Error1")." !");
	return;
    }
    $_=$paperName;
    if(m/ /) {

	messageBox( $transObj->translate("MSG_ERROR"),
		    $transObj->translate("DCPS_Dlg_Paper_Error4"));
	return;
    }
    # check if its unique
    # check in the custom paper Sizes Hash
    foreach my $customName (keys %$customPaperSizes ) {
	if(( $paperName eq $customName ) && ($paperName ne $helper->{"oldName"})) {
	    messageBox( $transObj->translate("MSG_ERROR"),
			$transObj->translate("DCPS_Dlg_Paper_Error2"));
	    return;
	}
    }
    # make the list with standard page  sizes names
    my @standardPaperSizes= $printer->enumFeatureOptions("*PageSize");
    #check in the list of the standard page sizes
    foreach my $standardName ( @standardPaperSizes) {
	if($paperName eq $standardName) {
	    messageBox( $transObj->translate("MSG_ERROR"),
			$transObj->translate("DCPS_Dlg_Paper_Error3"));
	    return;
	}
    }
    # get the values from the spin controls , 
    # convert them to the dots metrics and save them into 
    # the Paper Hash.
    # save the new defined custom page in the list
    my  %newCustomPaperSize=();
    my $width  = $widthSpin->get_value_as_float;
    my $length = $lengthSpin->get_value_as_float;

    my $dotsWidth  =  convertMetrics($helper->{metric},"dots",$width);
    my $dotsLength =  convertMetrics($helper->{metric},"dots",$length);

    $newCustomPaperSize{"Width"}   =  $dotsWidth;
    $newCustomPaperSize{"Length"}  =  $dotsLength;

    if( $helper->{"oldName"} ne "" )  {
	$customPaperSizes->{$helper->{"oldName"}}=undef;
    }

    $customPaperSizes->{$paperName}=\%newCustomPaperSize;


    # remove the current entry in the CList of the custom paper sizes and reload it 
    my $papersCList = $helper->{"papersCList"};
    my $row = $helper->{"row"};
    if(defined $row ) {
	my $oldRow = $$row;
	$papersCList->remove($$row);
	$papersCList->insert($oldRow ,$paperName);
	$papersCList->set_row_data($oldRow,\%newCustomPaperSize);

    }
    else {
	$papersCList->append($paperName);
	my $currentRow=$papersCList->rows-1;
	$papersCList->set_row_data($currentRow,\%newCustomPaperSize);
    }

    $helper->{"printer"}         = undef;
    $helper->{"customPaperSizes"}= undef;

    $helper->{"window"}           = undef;
    $helper->{"nameEntry"}        = undef; 
    $helper->{"widthSpin"}        = undef;
    $helper->{"widthLabel"}       = undef;
    $helper->{"lengthSpin"}       = undef;
    $helper->{"lengthLabel"}      = undef;

    $helper = undef;


    $window->destroy;
    
}

#################################################################################
#
#  subroutine:    cancelClicked      ( FOR LOCAL USE )
#   
#  description:   callback function for Cancel  button
#
#  parameters:    widget   - a ref to the Cancel  widget
#
#  return:        ( no value ) 
#
#################################################################################
sub cancelCustomPaperSizeDlgClicked {
    my $cancelButton = shift;
    my $helper       = $cancelButton->get_user_data;
    my $window       = $helper->{"window"};

    $helper->{"printer"}         = undef;
    $helper->{"customPaperSizes"}= undef;

    $helper->{"window"}           = undef;
    $helper->{"nameEntry"}        = undef; 
    $helper->{"widthSpin"}        = undef;
    $helper->{"widthLabel"}       = undef;
    $helper->{"lengthSpin"}       = undef;
    $helper->{"lengthLabel"}      = undef;

    $helper = undef;
   
    $window->destroy;

}



#################################################################################
#################################################################################
##
##  subroutine:   custompageListDlg  ( EXPORTED )
##   
##  description:  the dialog window for the list of the custom paper sizes
##
##  parameters:  1. cpList      - the custom pages Hash
##               2. printer     - the ref. to the printer assoc. with the queue 
##                  
##  return:      ( no value )
##
#################################################################################
#################################################################################
sub custompageListDlg {
    if ( @_ < 1 ) {
	die ("Error  :file ",__FILE__,":line ",
	     __LINE__," :function requires 2 parameter(s) .\n");
    }
    # custom pages List 
    my $customPaperSizes= shift;
    my $printer= shift;
    
    # the row selected in the CList of Paper Names
    my $row = -1;
    my @selectedRows = ();
    my %helper;
    $helper{"customPaperSizes"}=$customPaperSizes;
    $helper{"printer"}         =$printer;
    $helper{"row"}             =\$row;
    $helper{"selectedRows"}    =\@selectedRows;

######
#         DEFINE THE CONTAINTER WINDOW
#
    my $window = new Gtk::Window('toplevel');
    $helper{"window"}=$window;
    $window->set_user_data(\%helper);
    $window->set_title( $transObj->translate("CPS_Dlg_Title"));
    $window->set_position('center');
    $window->set_modal(1);
    $window->show;       
    $window->set_default_size(300,200);

#####
#       DEFINE THE TOP V. BOX THAT CONTAIS ALL CONTROLS
#
    my $topBox = new Gtk::VBox(0,5);
    $window->add($topBox);
    $topBox->show;
    $topBox->set_border_width(20);




#   label control for papers 
    my $papersLabel = new Gtk::Label($transObj->translate("CPS_Dlg_Label"));
    $papersLabel->show;
    $topBox->pack_start($papersLabel,1,1,1);


#   custom paper sizes CList 
    my $papersCList=new_with_titles Gtk::CList("    ");
    $helper{"papersCList"}=$papersCList;
    $papersCList->set_user_data(\%helper);
    $papersCList->signal_connect("select_row"  , \&paperRowSelected   );
    $papersCList->signal_connect("unselect_row", \&paperRowUnSelected   );
    $papersCList->set_selection_mode("multiple");
    $papersCList->column_titles_passive;

    $papersCList->show;
    $papersCList->column_titles_hide;
 
#   scroll window for papers list box control
    my $papersSW = new Gtk::ScrolledWindow(undef, undef);
    $papersSW->show;
    $papersSW->set_policy('automatic', 'automatic');
    $papersSW->add($papersCList);
    $papersSW->set_usize(100, 100);
    $topBox->pack_start($papersSW,0,0,1);


#####
#   DEFINE THE  H. BOX OF BUTTONS
#
    my $buttonsBox = new Gtk::HBox(1,1);
    $buttonsBox->show;
    $topBox->pack_start($buttonsBox,1,1,1);


#####
#      DEFINE THE BUTTONS
#
    # Add button
    my $addButton = new Gtk::Button($transObj->translate("add"));
    $helper{"addButton"} = $addButton;
    $addButton->signal_connect("clicked", \&addClickedListDlg);
    $addButton->set_user_data(\%helper);
    $addButton->show;
    $buttonsBox->pack_start($addButton,1,1,5);
    # Edit button
    my $editButton = new Gtk::Button($transObj->translate("edit"));
    $helper{"editButton"} = $editButton;
    $editButton->set_sensitive(0);
    $editButton->set_user_data(\%helper);
    $editButton->signal_connect("clicked", \&editClickedListDlg);
    $editButton->show;
    $buttonsBox->pack_start($editButton,1,1,5);
    # Delete button
    my $deleteButton = new Gtk::Button($transObj->translate("delete"));
    $helper{"deleteButton"} = $deleteButton;
    $deleteButton->set_user_data(\%helper);
    $deleteButton->set_sensitive(0);
    $deleteButton->signal_connect("clicked", \&deleteClickedListDlg);
    $deleteButton->show;
    $buttonsBox->pack_start($deleteButton,1,1,5);
    # Close button
    my $closeButton = new Gtk::Button($transObj->translate("CPS_Dlg_Done"));
    $helper{"closeButton"} = $closeButton;
    $closeButton->set_user_data(\%helper);
    $closeButton->signal_connect("clicked",\&closeClickedListDlg);
    $closeButton->show;
    $buttonsBox->pack_start($closeButton,1,1,5);

######
#      LOAD DATA IN THE LIST
#
    foreach my $customPage (keys %$customPaperSizes ) {
	if( defined $customPaperSizes->{$customPage} ) {
	    # add the name of the custom page in the list
	    $papersCList->append($customPage);
	    my $currentRow=$papersCList->rows-1;
	    # attach to the custom page the hash with it's values
	    $papersCList->set_row_data($currentRow,$customPaperSizes->{$customPage} );
	}
    }
}


sub paperRowSelected {
    my ( $widget , $row , $column , $event ) = @_;
    my $helper = $widget->get_user_data;
    my $editButton   = $helper->{"editButton"};
    my $deleteButton = $helper->{"deleteButton"};
    my $selectedRow = $helper->{"row"};
    my $selectedRows = $helper->{"selectedRows"};
    $$selectedRow = $row;


    @$selectedRows = (@$selectedRows,$row); 

    # get the number of the current selections and
    # set senzitivity of Edit and Delete buttons
    my $nrSelectedRows = @$selectedRows;
    if( $nrSelectedRows < 1 ) {
	$editButton->set_sensitive(0);
	$deleteButton->set_sensitive(0);
    } elsif (  $nrSelectedRows == 1 ) {
	$editButton->set_sensitive(1);
	$deleteButton->set_sensitive(1);
    } else {
	$editButton->set_sensitive(0);
	$deleteButton->set_sensitive(1);
    }



}

sub paperRowUnSelected {
    my ( $widget , $row , $column , $envent ) =@_;
    my $helper =  $widget->get_user_data;
    my $editButton   = $helper->{"editButton"};
    my $deleteButton = $helper->{"deleteButton"};
    my $selectedRow = $helper->{"row"};
    my $selectedRows = $helper->{"selectedRows"};

    # delete from the list the unselected row
    my @newList = ();
    foreach my $selRow (@$selectedRows) {
	if ( $selRow != $row) {
	    @newList = (@newList , $selRow ) ;
	} 
    }

    # set the single line selection index;
    if ( defined $newList[0] ) {
	$$selectedRow = $newList[0];
    } else {
	$$selectedRow = -1;
    }
    $helper->{"selectedRows"}=\@newList;
    
    # get the number of the current selections and
    # set senzitivity of Edit and Delete buttons
    my $nrSelectedRows = @newList;
    if( $nrSelectedRows < 1 ) {
	$editButton->set_sensitive(0);
	$deleteButton->set_sensitive(0);
    } elsif (  $nrSelectedRows == 1 ) {
	$editButton->set_sensitive(1);
	$deleteButton->set_sensitive(1);
    } else {
	$editButton->set_sensitive(0);
	$deleteButton->set_sensitive(1);
    }
    
}

sub deleteClickedListDlg {
    my $deleteButton = shift;
    my $helper       = $deleteButton->get_user_data;

    my $customPaperSizes = $helper->{"customPaperSizes"};
    my $papersCList      = $helper->{"papersCList"};

    my $editButton   = $helper->{"editButton"};
 
    my $selectedRow = $helper->{"row"};
    my $selectedRows = $helper->{"selectedRows"};

    foreach my $selRow ( @$selectedRows ) {
	my $nameRow = $papersCList->get_text($selRow,0);
	# delete form the custom Paper Sizes list
	$customPaperSizes->{ $nameRow } = undef;
    }
    @$selectedRows = ();
    $$selectedRow = -1;
    $editButton->set_sensitive(0);
    $deleteButton->set_sensitive(0);

    # clear the CList and reload it 
    $papersCList->clear;

######
#      LOAD DATA IN THE LIST
#
    foreach my $customPage (keys %$customPaperSizes ) {
	if ( defined $customPaperSizes->{$customPage} ) {
	    # add the name of the custom page in the list
	    $papersCList->append($customPage);
	    my $currentRow=$papersCList->rows-1;
	    # attach to the custom page the hash with it's values
	    $papersCList->set_row_data($currentRow,$customPaperSizes->{$customPage} );
	}
    }


}

#################################################################################
#
#  subroutine:    closeClickedListDlg      ( FOR LOCAL USE )
#   
#  description:   callback function for Cancel  button
#
#  parameters:    
#
#  return:        ( no value ) 
#
#################################################################################
sub closeClickedListDlg {
    my $closeButton = shift;
    my $helper       = $closeButton->get_user_data;
    my $window       = $helper->{"window"};
    # free the hash entry from controls



    $helper->{"customPaperSizes"} = undef;
    $helper->{"printer"}          = undef;
    $helper->{"row"}              = undef;
    $helper->{"selectedRows"}     = undef;
    $helper->{"window"}           = undef;
    $helper->{"papersCList"}      = undef;
    $helper->{"addButton"}        = undef;
    $helper->{"deleteButton"}     = undef;
    $helper->{"closeButton"}      = undef;
    $helper = undef;

    $window->destroy;

}

#################################################################################
#
#  subroutine:    addClickedListDlg      ( FOR LOCAL USE )
#   
#  description:   callback function for Add  button
#
#  parameters:   
#
#  return:        ( no value ) 
#
#################################################################################
sub addClickedListDlg {
    my $addButton = shift;
    my $helper           = $addButton->get_user_data;
    my $window           = $helper->{"window"};
    my $printer          = $helper->{"printer"};
    my $customPaperSizes = $helper->{"customPaperSizes"};
    my $papersCList      = $helper->{"papersCList"};
    my $name             = "";


    customPaperSizeDlg($name,undef,undef,$printer,$customPaperSizes,$papersCList,undef);

}


#################################################################################
#
#  subroutine:    editClickedListDlg      ( FOR LOCAL USE )
#   
#  description:   callback function for Edit  button
#
#  parameters:   
#
#  return:        ( no value ) 
#
#################################################################################
sub editClickedListDlg {
    my $editButton        = shift;
    my $helper            = $editButton->get_user_data;
    my $window            = $helper->{"window"};
    my $printer           = $helper->{"printer"};
    my $customPaperSizes  = $helper->{"customPaperSizes"};
    my $papersCList       = $helper->{"papersCList"};
    # get the current selection from the customPaperSizes ListBox
    my $row               = $helper->{"row"};
    my $name         = $papersCList->get_text($$row,0);
    my $userInfo     = $papersCList->get_row_data($$row);
    # get the current selection data
    my $width        = $userInfo->{"Width"};
    my $length       = $userInfo->{"Length"};


    # call customPaperSizeDlg :
    #  param1 :    the name of the Paper Size
    #  param2 :    width dimmension
    #  param3 :    length dimmension
    #  param4 :    the printer object
    #  param5 :    the list of the custom paper sizes
    #  param6 :    the CList with the Paper Sizes Names
    #  param7 :    the index of the paper size name in the list

    customPaperSizeDlg($name,$width,$length,$printer,$customPaperSizes,$papersCList,$row);

}










